###############################################################################
##  Bisque                                                                   ##
##  Center for Bio-Image Informatics                                         ##
##  University of California at Santa Barbara                                ##
## ------------------------------------------------------------------------- ##
##                                                                           ##
##     Copyright (c) 2007,2008,2009,2010,2011                                ##
##     by the Regents of the University of California                        ##
##                            All rights reserved                            ##
##                                                                           ##
## Redistribution and use in source and binary forms, with or without        ##
## modification, are permitted provided that the following conditions are    ##
## met:                                                                      ##
##                                                                           ##
##     1. Redistributions of source code must retain the above copyright     ##
##        notice, this list of conditions, and the following disclaimer.     ##
##                                                                           ##
##     2. Redistributions in binary form must reproduce the above copyright  ##
##        notice, this list of conditions, and the following disclaimer in   ##
##        the documentation and/or other materials provided with the         ##
##        distribution.                                                      ##
##                                                                           ##
##                                                                           ##
## THIS SOFTWARE IS PROVIDED BY <COPYRIGHT HOLDER> ''AS IS'' AND ANY         ##
## EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE         ##
## IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR        ##
## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> OR           ##
## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,     ##
## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,       ##
## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR        ##
## PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF    ##
## LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING      ##
## NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS        ##
## SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              ##
##                                                                           ##
## The views and conclusions contained in the software and documentation     ##
## are those of the authors and should not be interpreted as representing    ##
## official policies, either expressed or implied, of <copyright holder>.    ##
###############################################################################
"""
SYNOPSIS
========


DESCRIPTION
===========

"""
from __future__ import print_function
import optparse
import os
import posixpath
import sys
import logging
import urlparse
import urllib
import datetime


try:
    from lxml import etree
except ImportError:
    import xml.etree.ElementTree as etree

#from bq.config.environment import load_environment
#from bq.util import http
from bq.util.configfile import ConfigFile
from bq.util.paths import config_path
from bq.util.commands import find_site_cfg
from bq.util.urlnorm import norm
from bqapi import *


log = logging.getLogger('bq.engine.command.module_admin')

usage = """
%prog module [register|unregister/list-engine/list-server] [-u user:pass] [-r bisque_root_url]  [-p] [-a] http://myengnine.org/engine_service/[MyModule]
  Version 2

  Register or unregister and bisque analysis (Engine) on a bisque server.

  The module engine uri must be specified
  Optionaly include the the module_uri on the server to disambiguate

"""

def error (msg):
    print (msg, file=sys.stderr)


class module_admin(object):
    desc = 'module operations: register, unregister'
    def __init__(self, version):
        parser = optparse.OptionParser(usage=usage, version="%prog " + version)
        parser.add_option('-u', '--user', default=None, help='Login as user <user>:<pass>')
        parser.add_option('-a', '--all', action='store_true', help='Register/Unregister all modules at engine',
                          default=False)
        parser.add_option('-r', '--root', help='Bisque server root url')
        parser.add_option('-p', '--published', help='Make published module', default=False, action='store_true')
        parser.add_option('--cas', help='login with CAS', default=False, action='store_true')

        options, args = parser.parse_args()
        self.args = args
        self.options = options
        self.command = None
        self.root = None
        if len(self.args):
            self.command = getattr(self, self.args.pop(0).replace('-', '_'), None)
        if not self.command:
            parser.error("no valid command given")

        self.credentials = None
        if self.options.user:
            self.credentials = tuple(self.options.user.split(':'))


        self.module_uri = None
        if len(self.args):
            self.engine_path = self.args.pop(0)
            self.engine_path = posixpath.join (self.engine_path, '')

            if len(self.args):
                self.module_uri = self.args.pop(0)
        self.parser = parser



    def run(self):
        if self.options.root:
            self.root = self.options.root
        else:
            site_cfg = find_site_cfg('site.cfg')
            cfile = ConfigFile (site_cfg)
            print ("FILE :", site_cfg)

        if not self.root:
            print ("need a bisque server to contact.. please specify with -r ")

        try:
            self.command()
        except comm.BQCommError, e:
            error ("Problem communicating %s"% e)


    def create_session(self, root_required = True):
        if root_required and not self.root:
            self.parser.error ("need a bisque server to contact.. please specify with -r or update site.cfg")



        self.session = BQSession()
        if self.credentials:
            if self.options.cas:
                self.session.init_cas (self.credentials[0], self.credentials[1], bisque_root = self.root, create_mex=False)
            else:
                self.session.init_local (self.credentials[0], self.credentials[1], bisque_root = self.root, create_mex=False)



    def get_xml(self, url):
        print ("loading ", url)
        xml = self.session.fetchxml (url)
        return xml

    def get_modules(self, engine_path):
        'list module urls  at engine given path path'
        engine_path = norm(engine_path + '/')
        modules = self.get_xml( url = urlparse.urljoin(engine_path, '_services'))
        if modules is None:
            return error ('Cannot read modules from engine: %s' % engine_path)
        return  [ m.get('value') for m in modules ]

    def register_one(self, module_path):
        bisque_root = self.root
        module_path = norm(module_path + '/')
        module_register = norm (urlparse.urljoin(bisque_root, "module_service/register_engine") + '/')
        module_xml = self.get_xml( url = urlparse.urljoin(module_path, 'definition'))
        if module_xml is None:
            error ("cannot read definition from %s!  Is engine address correct?")
        name = module_xml.get('name')
        if module_xml is not None:
            log.info ("POSTING %s to %s" % (name, module_register))
            #engine = etree.Element ('engine', uri = module_path)
            #engine.append(module_xml)
            #xml =  etree.tostring (engine)
            #print xml
            module_xml.set('ts', datetime.datetime.now().isoformat())
            if self.options.published:
                for el in module_xml.getiterator(tag=etree.Element):
                    el.set ('permission', 'published')
            xml = etree.tostring(module_xml)
            params = [ ('engine_uri', module_path) ]
            if self.module_uri:
                params.append ( ('module_uri', module_uri) )
            url = "%s?%s" % (module_register, urllib.urlencode(params))
            self.session.postxml (url, xml = xml)
            print ( "Registered" )

    def register (self):
        self.create_session()
        if self.options.all:
            module_paths = self.get_modules(self.engine_path)
        else:
            module_paths = [ self.engine_path ]
        for m in module_paths:
            print ("registering %s" % m)
            self.register_one(m)


    def unregister(self):
        self.create_session()
        if self.options.all:
            module_paths = self.get_modules(self.engine_path)
        else:
            module_paths = [ self.engine_path ]
        for m in module_paths:
            print ("unregistering %s" % m)
            self.unregister_one(m)


    def unregister_one(self, module_path):
        bisque_root = self.root
        module_path = norm(module_path + '/')
        module_unregister = norm (urlparse.urljoin(bisque_root, "module_service/unregister_engine") + '/')

        module_name = os.path.basename(module_path.rstrip('/'))
        module_url = os.path.join (bisque_root, 'module_service', module_name)
        print ("unregister %s %s" % ( module_name, module_url))

        server_modules = self.get_xml( url = urlparse.urljoin(self.root, 'module_service'))
        for module_xml in server_modules:
            if module_xml.get ('name') == module_name:
                resource_uniq = module_xml.get ('resource_uniq')
                self.get_xml("%s?resource_uniq=%s" % (module_unregister, resource_uniq))
                print ("UnRegistered", module_name)
                return
        print ("No module found" , module_name)

        #module_name = module_path.split('/')[-2]
        #params = [ ('engine_uri', module_path) ]
        #if self.module_uri:
        #    params.append ( ('module_uri', module_uri) )

        #self.session.fetchxml (module_unregister, ** dict (params))


    def list_engine(self):
        self.create_session(root_required=False)
        module_paths = self.get_modules(self.engine_path)
        if module_paths is None:
            module_paths = self.get_modules(self.engine_path + '/engine_service/')
        for module in module_paths or []:
            print ( module )


    def list_server(self):
        self.create_session()
        from collections import namedtuple
        Row = namedtuple ('Row', ('name', 'engine', 'module'))
        server_modules = self.get_xml( url = urlparse.urljoin(self.root, 'module_service'))
        if server_modules is None:
            error ("No modules registered at %s. Is this a bisque server?" % self.root)

        rows = [ Row(name=module.get('name'), engine=module.get('value'), module=module.get ('uri'))
                 for module in server_modules ]
        pprinttable ( rows)


#taken from
#http://stackoverflow.com/questions/5909873/python-pretty-printing-ascii-tables
def pprinttable(rows):
  if len(rows) > 1:
    headers = rows[0]._fields
    lens = []
    for i in range(len(rows[0])):
      lens.append(len(max([x[i] for x in rows] + [headers[i]],key=lambda x:len(str(x)))))
    formats = []
    hformats = []
    for i in range(len(rows[0])):
      if isinstance(rows[0][i], int):
        formats.append("%%%dd" % lens[i])
      else:
        formats.append("%%-%ds" % lens[i])
      hformats.append("%%-%ds" % lens[i])
    pattern = " | ".join(formats)
    hpattern = " | ".join(hformats)
    separator = "-+-".join(['-' * n for n in lens])
    print (hpattern % tuple(headers))
    print (separator)
    for line in rows:
      print (pattern % tuple(line))
  elif len(rows) == 1:
    row = rows[0]
    hwidth = len(max(row._fields,key=lambda x: len(x)))
    for i in range(len(row)):
      print ("%*s = %s" % (hwidth,row._fields[i],row[i]))
